package issueData;

import com.fasterxml.jackson.databind.ObjectMapper;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

public class IssueRepository {
    private List<Issue> issues;

    public IssueRepository() {
        this.issues = new ArrayList<>();
        readAllIssues();
    }

    public List<Issue> getIssues() {
        return issues;
    }

    private void readAllIssues() {
        ObjectMapper mapper = new ObjectMapper();
        File[] directories = new File("/home/jack/Documents/testrepo/.issues/").listFiles(File::isDirectory);

        Arrays.stream(Objects.requireNonNull(directories)).forEach(issueDir -> {
            if (issueDir.getName().equals(".git")) return;

            try {
                String originalHash = issueDir.getName();
                String title = mapper.readValue(new File(issueDir.getAbsolutePath() + "/title"), String.class);
                String description = mapper.readValue(new File(issueDir.getAbsolutePath() + "/description"), String.class);
                String creator = mapper.readValue(new File(issueDir.getAbsolutePath() + "/creator"), String.class);
                boolean status = mapper.readValue(new File(issueDir.getAbsolutePath() + "/status"), boolean.class);
                long creationTime = mapper.readValue(new File(issueDir.getAbsolutePath() + "/creationTime"), long.class);
                long edited = mapper.readValue(new File(issueDir.getAbsolutePath() + "/editedTime"), long.class);

                Issue issue = new Issue();
                issue.setTitle(title);
                issue.setDescription(description);
                issue.setCreator(creator);
                issue.setOriginalHash(originalHash);
                issue.setStatus(status);
                issue.setCreationTime(creationTime);
                issue.setEdited(edited);

                String commentDirPath = "/home/jack/Documents/testrepo/.issues/" + issueDir.getName() + "/comments/";
                File[] commentDirs = new File(commentDirPath).listFiles((File::isDirectory));

                for (File commentDir : Objects.requireNonNull(commentDirs)) {
                    Comment comment = new Comment();
                    String commentHash = commentDir.getName();
                    String author = mapper.readValue(new File(commentDirPath + commentDir.getName() + "/author"), String.class);
                    String message = mapper.readValue(new File(commentDirPath + commentDir.getName() + "/message"), String.class);
                    long commentTime = mapper.readValue(new File(commentDirPath + commentDir.getName() + "/creationTime"), long.class);

                    comment.setHash(commentHash);
                    comment.setAuthor(author);
                    comment.setMessage(message);
                    comment.setCreationTime(commentTime);

                    issue.addComment(comment);
                }

                issues.add(issue);
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
    }


}
